<html>
  <body>
    <div id="event"></div>
    <script type="text/javascript">
      var eventOutputContainer = document.getElementById("event");
      var evtSrc = new EventSource("/subscribe/{{ destination }}");
      evtSrc.onmessage = function(e) {
         console.log(e.data);
         eventOutputContainer.innerHTML = e.data;
      };
    </script>
  </body>
</html>